# zarafa4ucsRole UDM hook 
#
# Copyright 2012 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

from univention.admin.hook import simpleHook

class zarafa4ucsRole(simpleHook):

	type = 'zarafa4ucsRole'

	def __isUsersUser(self, module):
		return 'username' in module.descriptions

	def __zarafaContact(self, module):
		if not hasattr(module, 'options'):
			return False

		# mail,person account and zarafa contact
		if module.get("zarafa-role") and module["zarafa-role"] == "contact":
			for option in module.options:
				if option == "mail" or option == "person":
					pass
				else:
					return False
			return True

		return False

	def __zarafaContactAcquireUidNumber(self, module, ml):

		# acquire uidNumber if object is a contact AND module has uidNum (the module "settings/usertemplate" does not contain uidNum ==> skip it!)
		if self.__isUsersUser(module) and self.__zarafaContact(module):
			if not module.get("uidNumber"):
				ml.append(('uidNumber', [module.uidNum]))
				module.alloc.append(('uidNumber', module.uidNum))
		return ml

	def __zarafaContactReleaseUidNumber(self, module):
		# the module "settings/usertemplate" does not contain uidNum ==> skip it!
		if not self.__isUsersUser(module):
			return

		# release uidNumber
		if self.__zarafaContact(module) and module.oldattr.get("uidNumber", [""])[0]:
			univention.admin.allocators.release(module.lo, module.position, 'uidNumber', module.oldattr["uidNumber"][0])

	def __zarafaContactConfirmUidNumber(self, module):
		# the module "settings/usertemplate" does not contain uidNum ==> skip it!
		if not self.__isUsersUser(module):
			return

		# confirm uidNumber
		if self.__zarafaContact(module) and module.uidNum:
			univention.admin.allocators.confirm(module.lo, module.position, 'uidNumber', module.uidNum)

		return

	def __zarafaRoles(self, module, ml):
		# if role has changed and module is not "settings/usertemplate"
		if self.__isUsersUser(module) and module.hasChanged("zarafa-role"):
	
			univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO,
				'zarafa4ucsRole: role has changed %s' % module["zarafa-role"])
	
			univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO,
				'zarafa4ucsRole: original modlist %s' % ml)
	
			univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO,
				'zarafa4ucsRole: oldattr %s' % module.oldattr)

			zarafaAccount = 0
			zarafaAdmin = 0
			zarafaSharedStoreOnly = 0
			zarafaContact = 0

			if module["zarafa-role"] == "user":
				zarafaAccount = 1
			elif module["zarafa-role"] == "admin":
				zarafaAccount = 1
				zarafaAdmin = 1
			elif module["zarafa-role"] == "store":
				zarafaSharedStoreOnly = 1
				zarafaAccount = 1
			elif module["zarafa-role"] == "contact":
				zarafaContact = 1
				zarafaAccount = 1
			else:
				pass

			# ignore non-zarafa users without primary mail address
			# and revert zarafa settings, except for contacts
			if not module["zarafa-role"] == "contact":
				if not module.get("mailPrimaryAddress") and module.oldattr.get("zarafa4ucsRole", ["none"])[0] == "none":
					newml = []
					zarafaAccount = 0
					zarafaAdmin = 0
					zarafaContact = 0
					for item in ml:
						if not item[0] == "zarafa4ucsRole":
							newml.append(item)
					newml.append(("zarafa4ucsRole", module["zarafa-role"], "none"))
					ml = newml
	
			# add/remove zarafa role
			ml.append((
				"zarafaAccount",
				module.oldattr.get("zarafaAccount", [""])[0],
				"%s" % zarafaAccount))
			ml.append((
				"zarafaAdmin",
				module.oldattr.get("zarafaAdmin", [""])[0],
				"%s" % zarafaAdmin))
			ml.append((
				"zarafaSharedStoreOnly",
				module.oldattr.get("zarafaSharedStoreOnly", [""])[0],
				"%s" % zarafaSharedStoreOnly))

			# add/remove objectClass zarafa-contact
			if zarafaContact:
				if not "zarafa-contact" in module.oldattr.get("objectClass", []):
					ml.append(("objectClass", "", "zarafa-contact"))
			else:
				if "zarafa-contact" in module.oldattr.get("objectClass", []):
					ml.append(("objectClass", "zarafa-contact", ""))
	
			univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO,
				'zarafa4ucsRole: changed modlist %s' % ml)
	
		return ml

	def hook_ldap_post_modify(self, module):
		pass
 
	def hook_open(self, module):
		pass
 
	def hook_ldap_pre_create(self, module):
		pass
 
	def hook_ldap_addlist(self, module, al=[]):
		al = self.__zarafaRoles(module, al)
		al = self.__zarafaContactAcquireUidNumber(module, al)
		return al
 
	def hook_ldap_post_create(self, module):
		self.__zarafaContactConfirmUidNumber(module)
		pass
 
	def hook_ldap_pre_modify(self, module):
		pass
 
	def hook_ldap_modlist(self, module, ml=[]):
		return self.__zarafaRoles(module, ml)
 
	def hook_ldap_pre_remove(self, module):
		pass
 
	def hook_ldap_post_remove(self, module):
		self.__zarafaContactReleaseUidNumber(module)
		pass
